@@ -38,7 +38,7 @@
 #              and not other keys.  Note that not all NXP processors
 #              including AHAB support this option.
 #
-#        Copyright 2018 NXP
+#        Copyright 2018-2022 NXP
 #
 #
 #-----------------------------------------------------------------------------
@@ -78,10 +78,11 @@ usage()
     echo "$0"
     echo
     echo "Command Line Mode:"
-    echo "$0 -existing-ca <y/n> [-ca-key <CA key name> -ca-cert <CA cert name>] -use-ecc <y/n> -kl <ECC/RSA Key Length> -da <digest algorithm> -duration <years> -srk-ca <y/n>"
+    echo "$0 -existing-ca <y/n> [-ca-key <CA key name> -ca-cert <CA cert name>] -kt <y/n> -kl <ECC/RSA Key Length> -da <digest algorithm> -duration <years> -srk-ca <y/n>"
     echo "Options:"
-    echo "    -kl: -use-ecc = y then Supported key lengths: p256, p384, p521"
-    echo "       : -use-ecc = n then Supported key lengths: 2048, 3072, 4096"
+    echo "    -kt ecc     : then Supported key lengths: p256, p384, p521"
+    echo "    -kt rsa     : then Supported key lengths: 2048, 3072, 4096"
+    echo "    -kt rsa-pss : then Supported key lengths: 2048, 3072, 4096"
     echo "    -da: Supported digest algorithms: sha256, sha384, sha512"
     echo
     echo "$0 -h | --help : This text"
@@ -137,9 +138,9 @@ then
                 ca_cert=$1
                 shift
                 ;;
-            -use-ecc)
+            -kt)
                 shift
-                use_ecc=$1
+                kt=$1
                 shift
                 ;;
             -kl)
@@ -187,10 +188,28 @@ fi
 
 if [ "$interactive" = "y" ]
 then
-    printf "Do you want to use Elliptic Curve Cryptography (y/n)?:  \b"
-    read use_ecc
+    printf "\n"
+    printf "Key type options (confirm targeted device supports desired key type):\n"
+    printf "Select the key type (possible values: rsa, rsa-pss, ecc)?:  \b"
+    read kt
 fi
 
+case $kt in
+    rsa)
+        rsa_algo='rsa'
+        use_ecc="n"
+        use_rsa_pss="n" ;;
+    rsa-pss)
+        rsa_algo='rsa-pss'
+        use_ecc="n"
+        use_rsa_pss="y" ;;
+    ecc)
+        use_ecc="y" ;;
+    *)
+        echo Invalid key type selected
+        exit 1 ;;
+esac
+
 if [ $use_ecc = "y" ]
 then
     if [ "$interactive" = "y" ]
@@ -203,7 +222,7 @@ then
     # Confirm that a valid key length has been entered
     case $kl in
         p256)
-            cn="prime256v1" ;;
+            cn="secp256r1" ;;
         p384)
             cn="secp384r1" ;;
         p521)
@@ -299,24 +318,24 @@ fi
 if [ ! -d "${crts_dir}" ]
 then
     echo ERROR: "Public keys directory ${crts_dir} is missing. Expecting <cst>/crts directory to be already created."
-    exit 1 
+    exit 1
 fi
 
 if [ ! -d "${ca_dir}" ]
 then
     echo ERROR: "Openssl configuration directory ${ca_dir} is missing. Expecting <cst>/ca directory to hold openssl configuration files."
-    exit 1 
+    exit 1
 fi
 
 # Switch current working directory to keys directory, if needed.
 if [ "${crt_dir}" != "${keys_dir}" ]
 then
-    cd "${keys_dir}" 
+    cd "${keys_dir}"
     if [ $? -ge 1 ]
     then
         echo ERROR: "Cannot change directory to ${keys_dir}"
         exit 1
-    fi 
+    fi
 fi
 
 # Check that the file "serial" is present, if not create it:
@@ -394,7 +413,7 @@ then
     openssl x509 -inform PEM -outform DER -in ${ca_cert}.pem -out ${ca_cert}.der
 
     # Cleanup
-    \rm temp_ca.pem
+    \rm temp_ca.pem $eck
 fi
 
 
@@ -414,8 +433,9 @@ then
         if [ $use_ecc = 'n' ]
         then
             # Generate SRK key
-            openssl genrsa -des3 -passout file:./key_pass.txt -f4 \
-                           -out ./temp_srk.pem ${kl}
+            openssl genpkey -algorithm ${rsa_algo} -pkeyopt rsa_keygen_bits:${kl} \
+                            -pass file:./key_pass.txt \
+                            -out ./temp_srk.pem
 
             srk_subj_req=/CN=SRK${i}_${da}_${kl}_65537_v3_usr/
             srk_crt=../crts/SRK${i}_${da}_${kl}_65537_v3_usr_crt
@@ -486,10 +506,12 @@ do
     echo
 
     if [ $use_ecc = 'n' ]
-        then
+    then
+        echo rsa algorithm = $rsa_algo
             # Generate SRK key
-            openssl genrsa -des3 -passout file:./key_pass.txt -f4 \
-                           -out ./temp_srk.pem ${kl}
+            openssl genpkey -algorithm ${rsa_algo} -pkeyopt rsa_keygen_bits:${kl} \
+                            -pass file:./key_pass.txt \
+                            -out ./temp_srk.pem
 
             srk_subj_req=/CN=SRK${i}_${da}_${kl}_65537_v3_ca/
             srk_crt=../crts/SRK${i}_${da}_${kl}_65537_v3_ca_crt
@@ -554,8 +576,9 @@ do
             srk_crt_i=../crts/SRK${i}_${da}_${kl}_65537_v3_ca_crt.pem
             srk_key_i=./SRK${i}_${da}_${kl}_65537_v3_ca_key.pem
             # Generate key
-            openssl genrsa -des3 -passout file:./key_pass.txt -f4 \
-                           -out ./temp_sgk.pem ${kl}
+            openssl genpkey -algorithm ${rsa_algo} -pkeyopt rsa_keygen_bits:${kl} \
+                            -pass file:./key_pass.txt \
+                            -out ./temp_sgk.pem
 
             sgk_subj_req=/CN=SGK${i}_1_${da}_${kl}_65537_v3_usr/
             sgk_crt=../crts/SGK${i}_1_${da}_${kl}_65537_v3_usr_crt
@@ -617,7 +640,7 @@ fi
 # Switch back to initial working directory, if needed.
 if [ "${crt_dir}" != "${keys_dir}" ]
 then
-    cd "${crt_dir}" 
+    cd "${crt_dir}"
     if [ $? -ge 1 ]
     then
         echo ERROR: "Cannot change directory to ${crt_dir}"
